import os
import unittest
from decimal import Decimal, InvalidOperation
import json

try:
    # python 3.4+ should use builtin unittest.mock not mock package
    from unittest.mock import patch
    from unittest import mock
except ImportError:
    from mock import patch
    import mock

import sys

from envs import env
from envs.exceptions import EnvsValueException

class EnvTestCase(unittest.TestCase):
    def setUp(self):
        # Integer
        os.environ.setdefault('VALID_INTEGER', '1')
        os.environ.setdefault('INVALID_INTEGER', '["seven"]')
        # String
        os.environ.setdefault('VALID_STRING', 'seven')
        # Boolean
        os.environ.setdefault('VALID_BOOLEAN', 'True')
        os.environ.setdefault('VALID_BOOLEAN_FALSE', 'false')
        os.environ.setdefault('INVALID_BOOLEAN', 'seven')
        # List
        os.environ.setdefault('VALID_LIST', "['1','2','3']")
        os.environ.setdefault('INVALID_LIST', "1")
        # Tuple
        os.environ.setdefault('VALID_TUPLE', "('True','FALSE')")
        os.environ.setdefault('INVALID_TUPLE', '1')
        # Dict
        os.environ.setdefault('VALID_DICT', "{'first_name':'Suge'}")
        os.environ.setdefault('INVALID_DICT', 'Aaron Rogers')
        # Float
        os.environ.setdefault('VALID_FLOAT', "5.0")
        os.environ.setdefault('INVALID_FLOAT', '[5.0]')
        # Decimal
        os.environ.setdefault('VALID_DECIMAL', "2.39")
        os.environ.setdefault('INVALID_DECIMAL', "FOOBAR")

    def test_integer_valid(self):
        self.assertEqual(1, env('VALID_INTEGER', var_type='integer'))

    def test_integer_invalid(self):
        with self.assertRaises(TypeError) as vm:
            env('INVALID_INTEGER', var_type='integer')

    def test_wrong_var_type(self):
        with self.assertRaises(ValueError) as vm:
            env('INVALID_INTEGER', var_type='set')

    def test_string_valid(self):
        self.assertEqual('seven', env('VALID_STRING'))

    def test_boolean_valid(self):
        self.assertEqual(True, env('VALID_BOOLEAN', var_type='boolean'))

    def test_boolean_valid_false(self):
        self.assertEqual(False, env('VALID_BOOLEAN_FALSE', var_type='boolean'))

    def test_boolean_invalid(self):
        with self.assertRaises(ValueError) as vm:
            env('INVALID_BOOLEAN', var_type='boolean')

    def test_list_valid(self):
        self.assertEqual(['1', '2', '3'], env('VALID_LIST', var_type='list'))

    def test_list_invalid(self):
        with self.assertRaises(TypeError) as vm:
            env('INVALID_LIST', var_type='list')

    def test_tuple_valid(self):
        self.assertEqual(('True', 'FALSE'), env('VALID_TUPLE', var_type='tuple'))

    def test_tuple_invalid(self):
        with self.assertRaises(TypeError) as vm:
            env('INVALID_TUPLE', var_type='tuple')

    def test_dict_valid(self):
        self.assertEqual({'first_name': 'Suge'}, env('VALID_DICT', var_type='dict'))

    def test_dict_invalid(self):
        with self.assertRaises(SyntaxError) as vm:
            env('INVALID_DICT', var_type='dict')

    def test_float_valid(self):
        self.assertEqual(5.0, env('VALID_FLOAT', var_type='float'))

    def test_float_invalid(self):
        with self.assertRaises(TypeError) as vm:
            env('INVALID_FLOAT', var_type='float')

    def test_decimal_valid(self):
        self.assertEqual(Decimal('2.39'), env('VALID_DECIMAL', var_type='decimal'))

    def test_decimal_invalid(self):
        with self.assertRaises(ArithmeticError) as vm:
            env('INVALID_DECIMAL', var_type='decimal')

    def test_defaults(self):
        self.assertEqual(env('HELLO', 5, var_type='integer'), 5)
        self.assertEqual(env('HELLO', 5.0, var_type='float'), 5.0)
        self.assertEqual(env('HELLO', [], var_type='list'), [])
        self.assertEqual(env('HELLO', {}, var_type='dict'), {})
        self.assertEqual(env('HELLO', (), var_type='tuple'), ())
        self.assertEqual(env('HELLO', 'world'), 'world')
        self.assertEqual(env('HELLO', False, var_type='boolean'), False)
        self.assertEqual(env('HELLO', 'False', var_type='boolean'), False)
        self.assertEqual(env('HELLO', 'true', var_type='boolean'), True)
        self.assertEqual(env('HELLO', Decimal('3.14'), var_type='decimal'), Decimal('3.14'))

    def test_without_defaults_allow_none(self):
        self.assertEqual(env('HELLO'), None)
        self.assertEqual(env('HELLO', var_type='integer'), None)
        self.assertEqual(env('HELLO', var_type='float'), None)
        self.assertEqual(env('HELLO', var_type='list'), None)

    def test_without_defaults_disallow_none(self):
        with self.assertRaises(EnvsValueException):
            env('HELLO', allow_none=False)
        with self.assertRaises(EnvsValueException):
            env('HELLO', var_type='integer', allow_none=False)
        with self.assertRaises(EnvsValueException):
            env('HELLO', var_type='float', allow_none=False)
        with self.assertRaises(EnvsValueException):
            env('HELLO', var_type='list', allow_none=False)

    def test_empty_values(self):
        os.environ.setdefault('EMPTY', '')
        self.assertEqual(env('EMPTY'), '')
        with self.assertRaises(SyntaxError):
            env('EMPTY', var_type='integer')
        with self.assertRaises(SyntaxError):
            env('EMPTY', var_type='float')
        with self.assertRaises(SyntaxError):
            env('EMPTY', var_type='list')
        with self.assertRaises(SyntaxError):
            env('EMPTY', var_type='dict')
        with self.assertRaises(SyntaxError):
            env('EMPTY', var_type='tuple')
        with self.assertRaises(ValueError):
            env('EMPTY', var_type='boolean')
        with self.assertRaises(ArithmeticError):
            env('EMPTY', var_type='decimal')

'''
Each CLI Test must be run outside of test suites in isolation
since Click CLI Runner alters the global context
'''
def setup_CliRunner(test_func):
    '''
    Decorator to initialize environment for CliRunner.
    '''
    def wrapper():
        from click.testing import CliRunner
        try:
            from cli import envs as cli_envs
        except ImportError:
            from .cli import envs as cli_envs

        try:
            from cli import envs
        except ImportError:
            from .cli import envs

        test_func()

    return wrapper

@mock.patch.object(sys, 'argv', ["list-envs"])
@setup_CliRunner
def test_list_envs():
    os.environ.setdefault('DEBUG', 'True')

    runner = CliRunner()
    result = runner.invoke(envs, ['list-envs', '--settings-file', 'envs.test_settings', '--keep-result', 'True'], catch_exceptions=False)

    output_expected = [{"default": None, "value": None, "var_type": "string", "key": "DATABASE_URL"},{"default": False, "value": "True", "var_type": "boolean", "key": "DEBUG"},{"default": [], "value": None, "var_type": "list", "key": "MIDDLEWARE"},{}]

    with open('.envs_result', 'r') as f:
        output_actual = json.load(f)

    exit_code_expected = 0
    exit_code_actual = result.exit_code


    assert exit_code_actual == exit_code_expected
    assert output_actual == output_expected





if __name__ == '__main__':
    unittest.main()




import ast
import json
import os
import sys
from decimal import Decimal

from .exceptions import EnvsValueException

_envs_list = []

class CLIArguments():
    LIST_ENVS = 'list-envs'
    CHECK_ENVS = 'check-envs'
    CONVERT_SETTINGS = 'convert-settings'

ARGUMENTS = CLIArguments()

ENVS_RESULT_FILENAME = '.envs_result'


def validate_boolean(value):
    true_vals = ('True', 'true', 1, '1')
    false_vals = ('False', 'false', 0, '0')
    if value in true_vals:
        value = True
    elif value in false_vals:
        value = False
    else:
        raise ValueError('This value is not a boolean value.')
    return value


class Env(object):
    valid_types = {
        'string': None,
        'boolean': validate_boolean,
        'list': list,
        'tuple': tuple,
        'integer': int,
        'float': float,
        'dict': dict,
        'decimal': Decimal
    }

    def __call__(self, key, default=None, var_type='string', allow_none=True):
        if ARGUMENTS.LIST_ENVS in sys.argv or ARGUMENTS.CHECK_ENVS in sys.argv:
            with open(ENVS_RESULT_FILENAME, 'a') as f:
                json.dump({'key': key, 'var_type': var_type, 'default': default, 'value': os.getenv(key)}, f)
                f.write(',')
        value = os.getenv(key, default)
        if not var_type in self.valid_types.keys():
            raise ValueError(
                'The var_type argument should be one of the following {0}'.format(
                    ','.join(self.valid_types.keys())))
        if value is None:
            if not allow_none:
                raise EnvsValueException('{}: Environment Variable Not Set'.format(key))
            return value
        return self.validate_type(value, self.valid_types[var_type], key)

    def validate_type(self, value, klass, key):
        if not klass:
            return value
        if klass in (validate_boolean, Decimal):
            return klass(value)
        if isinstance(value, klass):
            return value
        return klass(ast.literal_eval(value))


env = Env()



import ast
import importlib
import json
import os
import sys

from . import Env, ENVS_RESULT_FILENAME

VAR_TYPES = Env.valid_types.keys()

if sys.version_info >= (3, 0):
    raw_input = input

def import_util(imp):
    """
    Lazily imports a utils (class,
    function,or variable) from a module) from
    a string.
    @param imp:
    """

    mod_name, obj_name = imp.rsplit('.', 1)
    mod = importlib.import_module(mod_name)
    return getattr(mod, obj_name)


def convert_module(module):
    attr_list = []
    for k, v in module.__dict__.items():
        if k.isupper():
            convert = bool(int(raw_input('Convert {}? (1=True,0=False): '.format(k))))
            attr_dict = {'name': k, 'convert': convert}
            default_val = None
            if convert:

                default_val = raw_input('Default Value? (default: {}): '.format(v))
                if default_val:
                    default_val = ast.literal_eval(default_val)
                if not default_val:
                    default_val = v
                attr_dict['default_val'] = default_val

                var_type = raw_input('Variable Type Choices (ex. boolean,string,list,tuple,integer,float,dict): ')
                if not var_type in VAR_TYPES:
                    raise ValueError('{} not in {}'.format(var_type, VAR_TYPES))
                attr_dict['var_type'] = var_type
            if not default_val:
                default_val = v
            attr_list.append(attr_dict)
    return attr_list


def import_mod(module):
    if sys.version_info.major == 3:
        try:
            m = importlib.import_module(module)
        except ModuleNotFoundError:
            sys.path.insert(0, os.getcwd())
            m = importlib.import_module(module)
    else:
        try:
            m = importlib.import_module(module)
        except ImportError:
            sys.path.insert(0, os.getcwd())
            m = importlib.import_module(module)
    return m


def list_envs_module(module):
    with open(ENVS_RESULT_FILENAME, 'w+') as f:
        f.write('[')
    import_mod(module)
    with open(ENVS_RESULT_FILENAME, 'a') as f:
        f.write('{}]')
    with open(ENVS_RESULT_FILENAME, 'r') as f:
        envs_result = json.load(f)
        envs_result.pop()
    return envs_result



import os

import click
import jinja2
from terminaltables import AsciiTable

from envs.exceptions import EnvsValueException
from . import env
from . import ARGUMENTS, ENVS_RESULT_FILENAME
from .util import convert_module, import_mod, list_envs_module, raw_input

SETTINGS_TEMPLATE = jinja2.Environment(loader=jinja2.PackageLoader(
    'envs', 'templates')).get_template('settings.jinja2')


@click.group()
def envs():
    pass


@envs.command(ARGUMENTS.CONVERT_SETTINGS, help='Converts an existing settings file so it uses envs.')
@click.option('--settings-file', prompt=True,
              help='Settings Module? ex. settings or yourapp.settings')
def convert_settings(settings_file):
    attr_list = convert_module(import_mod(settings_file))
    template = SETTINGS_TEMPLATE.render(attr_list=attr_list)
    new_settings_filename = raw_input(
        'What should we name this new settings file? (DO NOT NAME IT THE SAME AS THE ORIGINAL BECAUSE THERE ARE NO IMPORTS): ')
    if not new_settings_filename:
        raise ValueError('Settings filename is required.')
    with open(new_settings_filename, 'w+') as f:
        f.write(template)
    click.echo(click.style('Your new settings file {}'.format(new_settings_filename), fg='green'))


@envs.command(ARGUMENTS.LIST_ENVS, help='Shows a list of env instances set in a settings file.')
@click.option('--settings-file', prompt=True,
              help='Settings Module? ex. settings or yourapp.settings')
@click.option('--keep-result', prompt=True, help='Keep the result file ({})?'.format(ENVS_RESULT_FILENAME), default=False)
def list_envs(settings_file, keep_result):
    envs_result = list_envs_module(settings_file)
    table_data = [
        ['Env Var', 'Var Type', 'Has Default', 'Environment Value'],

    ]
    table_data.extend(
        [[row.get('key'), row.get('var_type'), bool(row.get('default')), row.get('value')] for row in envs_result])
    click.echo(AsciiTable(table_data).table)

    if not keep_result:
        os.remove(ENVS_RESULT_FILENAME)


@envs.command(ARGUMENTS.CHECK_ENVS, help='Make sure that the defined envs with no default value have a value set in the environment.')
@click.option('--settings-file', prompt=True,
              help='Settings Module? ex. settings or yourapp.settings')
def check_envs(settings_file):
    envs_result = list_envs_module(settings_file)
    for row in envs_result:
        value = env(row.get('key'), row.get('default'), var_type=row.get('var_type'))
        if not value:
            raise EnvsValueException('{}: Environment Variable Not Set'.format(row.get('key')))
    os.remove(ENVS_RESULT_FILENAME)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['envs']

package_data = \
{'': ['*'], 'envs': ['templates/*']}

extras_require = \
{':extra == "cli"': ['click[cli]>=8.0.3,<9.0.0',
                     'Jinja2[cli]>=3.0.3,<4.0.0',
                     'terminaltables[cli]>=3.1.10,<4.0.0']}

entry_points = \
{'console_scripts': ['envs = envs.cli:envs']}

setup_kwargs = {
    'name': 'envs',
    'version': '1.4',
    'description': 'Easy access of environment variables from Python with support for strings, booleans, list, tuples, and dicts.',
    'long_description': None,
    'author': 'Brian Jinwright',
    'author_email': 'bjinwright@qwigo.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
